package com.gci.boot.modules.system.util;

import com.auth0.jwt.JWT;
import com.auth0.jwt.JWTCreator;
import com.auth0.jwt.JWTVerifier;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.exceptions.JWTDecodeException;
import com.auth0.jwt.interfaces.Claim;
import com.auth0.jwt.interfaces.DecodedJWT;
import com.gci.boot.constant.DataBaseConstant;
import com.gci.boot.common.util.SpringContextUtil;
import com.gci.boot.exception.GciBootException;
import com.gci.boot.modules.system.vo.LoginUser;
import com.gci.boot.modules.system.vo.SysUserCacheInfo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.formula.functions.T;
import org.apache.shiro.SecurityUtils;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.lang.reflect.Field;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Author Scott
 * @Date 2018-07-12 14:23
 * @Desc JWT工具类
 **/
@Slf4j
public class JwtUtil {

	// Token过期时间30分钟（用户登录过期时间是此时间的两倍，以token在reids缓存时间为准）
	public static long DEFAULT_EXPIRE_TIME = 30 * 60 * 1000;

	public static String secret;

	public void setExpireTime(long expireTime) {
		JwtUtil.DEFAULT_EXPIRE_TIME = expireTime;
	}

	public void setSecret(String secret){
		JwtUtil.secret = secret;
	}

	/**
	 * 校验token是否正确
	 *
	 * @param token  密钥
	 * @param secret 用户的密码
	 * @return 是否正确
	 */
	public static boolean verify(String token, String username, String secret) {
		try {
			// 根据密码生成JWT效验器
			Algorithm algorithm = Algorithm.HMAC256(secret);
			JWTVerifier verifier = JWT.require(algorithm).withClaim("username", username).build();
			// 效验TOKEN
			DecodedJWT jwt = verifier.verify(token);
			return true;
		} catch (Exception exception) {
			return false;
		}
	}

	/**
	 * 校验token是否正确
	 *
	 * @param token  密钥
	 * @return 是否正确
	 */
	public static boolean verify(String token, String username) {
		return verify(token, username, secret);
	}

	/**
	 * 获得token中的信息无需secret解密也能获得
	 *
	 * @return token中包含的用户名
	 */
	public static String getUsername(String token) {
		try {
			DecodedJWT jwt = JWT.decode(token);
			return jwt.getClaim("username").asString();
		} catch (JWTDecodeException e) {
			return null;
		}
	}

	public static <T> T getObject(String token, Class<T> tClass) {
		try {
			DecodedJWT jwt = JWT.decode(token);
			T t = tClass.newInstance();
			Field[] fields = tClass.getDeclaredFields();
			for(Field f:fields){
				f.setAccessible(true);
				String name = f.getName();
				Class<?> type = f.getType();
				if("java.lang.String".equals(type.getName())){
					f.set(t,jwt.getClaim(name).asString());
				}else if("java.Util.Date".equals(type.getName())){
					f.set(t,jwt.getClaim(name).asDate());
				}else if("java.lang.Integer".equals(type.getName()) || "int".equals(type.getName())) {
					f.set(t, jwt.getClaim(name).asInt());
				}else if("java.lang.Long".equals(type.getName()) || "long".equals(type.getName())) {
					f.set(t, jwt.getClaim(name).asLong());
				}else if("java.lang.Double".equals(type.getName()) || "double".equals(type.getName())) {
					f.set(t, jwt.getClaim(name).asDouble());
				}else if("java.lang.Boolean".equals(type.getName()) || "boolean".equals(type.getName())) {
					f.set(t, jwt.getClaim(name).asBoolean());
				}else if("java.util.List".equals(type.getName())) {
					Type genericType = f.getGenericType();
					Type[] typeArguments = ((ParameterizedType) genericType).getActualTypeArguments();
					f.set(t, jwt.getClaim(name).asList((Class)typeArguments[0]));
				}else if("java.util.Map".equals(type.getName())) {
					f.set(t, jwt.getClaim(name).asMap());
				}
				f.setAccessible(false);
			}
			return t;
		} catch (Exception e) {
			log.error("", e);
			return null;
		}
	}


	/**
	 * 生成签名，使用配置密钥，默认过期时间
	 * @param username
	 * @return
	 */
	public static String sign(String username) {
		return sign(username, secret, DEFAULT_EXPIRE_TIME);
	}

	/**
	 * 生成签名，使用默认过期时间
	 *
	 * @param username 用户名
	 * @param secret   用户的密码
	 * @return 加密的token
	 */
	public static String sign(String username, String secret) {
		return sign(username, secret, DEFAULT_EXPIRE_TIME);
	}

	/**
	 * 生成签名,自定义过期时间
	 * @param username
	 * @param secret
	 * @param expire
	 * @return
	 */
	public static String sign(String username, String secret, long expire) {
		Date date = new Date(System.currentTimeMillis() + expire);
		Algorithm algorithm = Algorithm.HMAC256(secret);
		// 附带username信息
		return JWT.create().withClaim("username", username).withExpiresAt(date).sign(algorithm);
	}

	/**
	 * 生成签名,自定义过期时间
	 * @param claims
	 * @param secret
	 * @param expire
	 * @return
	 */
	public static String sign(Map<String, Object> claims, String secret, long expire) {
		Date date = new Date(System.currentTimeMillis() + expire);
		Algorithm algorithm = Algorithm.HMAC256(secret);
		// 附带信息
		JWTCreator.Builder builder = JWT.create();
		if(claims != null){
			claims.forEach((k, v)->{
				if(v instanceof String){
					builder.withClaim(k, (String)v);
				}else if(v instanceof Integer){
					builder.withClaim(k, (Integer)v);
				}else if(v instanceof Date){
					builder.withClaim(k, (Date)v);
				}else if(v instanceof Long){
					builder.withClaim(k, (Long)v);
				}else if(v instanceof Double){
					builder.withClaim(k, (Double)v);
				}else if(v instanceof Boolean){
					builder.withClaim(k, (boolean)v);
				}
			});
		}
		return builder.withExpiresAt(date).sign(algorithm);
	}

	/**
	 * 生成签名,自定义过期时间
	 * @param claims
	 * @param secret
	 * @return
	 */
	public static String sign(Map<String, Object> claims, String secret) {
		return sign(claims, secret, DEFAULT_EXPIRE_TIME);
	}

	/**
	 * 生成签名,自定义过期时间
	 * @param claims
	 * @return
	 */
	public static String sign(Map<String, Object> claims) {
		return sign(claims, secret, DEFAULT_EXPIRE_TIME);
	}

	/**
	 * 根据request中的token获取用户账号
	 * 
	 * @param request
	 * @return
	 * @throws GciBootException
	 */
	public static String getUserNameByToken(HttpServletRequest request) throws GciBootException {
		String accessToken = request.getHeader("X-Access-Token");
		String username = getUsername(accessToken);
		if (StringUtils.isBlank(username)) {
			throw new GciBootException("未获取到用户");
		}
		return username;
	}
	
	/**
	  *  从session中获取变量
	 * @param key
	 * @return
	 */
	public static String getSessionData(String key) {
		//${myVar}%
		//得到${} 后面的值
		String moshi = "";
		if(key.indexOf("}")!=-1){
			 moshi = key.substring(key.indexOf("}")+1);
		}
		String returnValue = null;
		if (key.contains("#{")) {
			key = key.substring(2,key.indexOf("}"));
		}
		if (StringUtils.isNotBlank(key)) {
			HttpSession session = SpringContextUtil.getHttpServletRequest().getSession();
			returnValue = (String) session.getAttribute(key);
		}
		//结果加上${} 后面的值
		if(returnValue!=null){returnValue = returnValue + moshi;}
		return returnValue;
	}
	
	/**
	  * 从当前用户中获取变量
	 * @param key
	 * @param user
	 * @return
	 */
	//TODO 急待改造 sckjkdsjsfjdk
	public static String getUserSystemData(String key, SysUserCacheInfo user) {
		if(user==null) {
			user = GciDataAutorUtils.loadUserInfo();
		}
		//#{sys_user_code}%
		
		// 获取登录用户信息
		LoginUser sysUser = (LoginUser) SecurityUtils.getSubject().getPrincipal();
		
		String moshi = "";
		if(key.indexOf("}")!=-1){
			 moshi = key.substring(key.indexOf("}")+1);
		}
		String returnValue = null;
		//针对特殊标示处理#{sysOrgCode}，判断替换
		if (key.contains("#{")) {
			key = key.substring(2,key.indexOf("}"));
		} else {
			key = key;
		}
		//替换为系统登录用户帐号
		if (key.equals(DataBaseConstant.SYS_USER_CODE)|| key.toLowerCase().equals(DataBaseConstant.SYS_USER_CODE_TABLE)) {
			if(user==null) {
				returnValue = sysUser.getUsername();
			}else {
				returnValue = user.getSysUserCode();
			}
		}
		//替换为系统登录用户真实名字
		else if (key.equals(DataBaseConstant.SYS_USER_NAME)|| key.toLowerCase().equals(DataBaseConstant.SYS_USER_NAME_TABLE)) {
			if(user==null) {
				returnValue = sysUser.getRealname();
			}else {
				returnValue = user.getSysUserName();
			}
		}
		
		//替换为系统用户登录所使用的机构编码
		else if (key.equals(DataBaseConstant.SYS_ORG_CODE)|| key.toLowerCase().equals(DataBaseConstant.SYS_ORG_CODE_TABLE)) {
			if(user==null) {
				returnValue = sysUser.getOrgCode();
			}else {
				returnValue = user.getSysOrgCode();
			}
		}
		//替换为系统用户所拥有的所有机构编码
		else if (key.equals(DataBaseConstant.SYS_MULTI_ORG_CODE)|| key.toLowerCase().equals(DataBaseConstant.SYS_MULTI_ORG_CODE_TABLE)) {
			if(user.isOneDepart()) {
				returnValue = user.getSysMultiOrgCode().get(0);
			}else {
				returnValue = user.getSysMultiOrgCode().stream().collect(Collectors.joining(","));
			}
		}
		//替换为当前系统时间(年月日)
		else if (key.equals(DataBaseConstant.SYS_DATE)|| key.toLowerCase().equals(DataBaseConstant.SYS_DATE_TABLE)) {
			returnValue = user.getSysDate();
		}
		//替换为当前系统时间（年月日时分秒）
		else if (key.equals(DataBaseConstant.SYS_TIME)|| key.toLowerCase().equals(DataBaseConstant.SYS_TIME_TABLE)) {
			returnValue = user.getSysTime();
		}
		//流程状态默认值（默认未发起）
		else if (key.equals(DataBaseConstant.BPM_STATUS)|| key.toLowerCase().equals(DataBaseConstant.BPM_STATUS_TABLE)) {
			returnValue = "1";
		}
		if(returnValue!=null){returnValue = returnValue + moshi;}
		return returnValue;
	}
	
	public static void main(String[] args) {
		 String token = "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJleHAiOjE1NjUzMzY1MTMsInVzZXJuYW1lIjoiYWRtaW4ifQ.xjhud_tWCNYBOg_aRlMgOdlZoWFFKB_givNElHNw3X0";
		 System.out.println(JwtUtil.getUsername(token));
	}
}
